/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * KryFontSelection widget for Gtk+, by Damon Chaplin, May 1998.
 * Based on the GnomeFontSelector widget, by Elliot Lee, but major changes.
 * The GnomeFontSelector was derived from app/text_tool.c in the GIMP.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

#ifndef __GTK_KRYFONTSEL_H__
#define __GTK_KRYFONTSEL_H__


#include <gdk/gdk.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkvbox.h>

#define GTK_TYPE_KRY_FONT_SELECTION              (kry_font_selection_get_type ())
#define KRY_FONT_SELECTION(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_KRY_FONT_SELECTION, KryFontSelection))
#define KRY_FONT_SELECTION_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_KRY_FONT_SELECTION, KryFontSelectionClass))
#define GTK_IS_KRY_FONT_SELECTION(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_KRY_FONT_SELECTION))
#define GTK_IS_KRY_FONT_SELECTION_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_KRY_FONT_SELECTION))
#define KRY_FONT_SELECTION_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_KRY_FONT_SELECTION, KryFontSelectionClass))


#define GTK_TYPE_KRY_FONT_SELECTION_DIALOG              (kry_font_selection_dialog_get_type ())
#define KRY_FONT_SELECTION_DIALOG(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_KRY_FONT_SELECTION_DIALOG, KryFontSelectionDialog))
#define KRY_FONT_SELECTION_DIALOG_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_KRY_FONT_SELECTION_DIALOG, KryFontSelectionDialogClass))
#define GTK_IS_KRY_FONT_SELECTION_DIALOG(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_KRY_FONT_SELECTION_DIALOG))
#define GTK_IS_KRY_FONT_SELECTION_DIALOG_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_KRY_FONT_SELECTION_DIALOG))
#define KRY_FONT_SELECTION_DIALOG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_KRY_FONT_SELECTION_DIALOG, KryFontSelectionDialogClass))


typedef struct _KryFontSelection	     KryFontSelection;
typedef struct _KryFontSelectionClass	     KryFontSelectionClass;

typedef struct _KryFontSelectionDialog	     KryFontSelectionDialog;
typedef struct _KryFontSelectionDialogClass  KryFontSelectionDialogClass;

struct _KryFontSelection
{
  GtkVBox parent_instance;

  struct fontsel_info *fontsel_info;
  
  GtkWidget *font_entry;
  GtkWidget *family_list;
  GtkWidget *font_style_entry;
  GtkWidget *face_list;
  GtkWidget *size_entry;
  GtkWidget *size_list;
  GtkWidget *pixels_button;
  GtkWidget *points_button;
  GtkWidget *filter_button;
  GtkWidget *preview_entry;

  GtkToggleButton *toggle_bold;
  GtkToggleButton *toggle_italic;
  GtkToggleButton *toggle_underline;
  GtkToggleButton *toggle_strikethrough;

  GtkDrawingArea *preview_area;
  GtkComboBox *combo_style;

  GtkCheckButton *toggle_video_preview;

  char *orig_style;

  int bold;
  int italic;
  int strikethrough;
  int underline;

  int orig_bold;
  int orig_italic;
  int orig_strikethrough;
  int orig_underline;
  int orig_font_size;
  char *orig_font_name;

  char *affected_style;

  GList *fonts;

  PangoFontFamily *family;	/* Current family */
//#ifdef _WINDOWS
  char *face;
//#else
//  PangoFontFace *face;		/* Current face */
//#endif
  
  gint size;
  
  GdkFont *font;		/* Cache for gdk_font_selection_get_font, so we can preserve
				 * refcounting behavior
				 */
};

struct _KryFontSelectionClass
{
  GtkVBoxClass parent_class;

  /* Padding for future expansion */
  void (*_gtk_reserved1) (void);
  void (*_gtk_reserved2) (void);
  void (*_gtk_reserved3) (void);
  void (*_gtk_reserved4) (void);
};


struct _KryFontSelectionDialog
{
  GtkDialog parent_instance;

  /*< private >*/
  GtkWidget *fontsel;

  GtkWidget *main_vbox;
  GtkWidget *action_area;
  /*< public >*/
  GtkWidget *ok_button;
  GtkWidget *apply_button;
  GtkWidget *cancel_button;

  /*< private >*/

  /* If the user changes the width of the dialog, we turn auto-shrink off.
   * (Unused now, autoshrink doesn't mean anything anymore -Yosh)
   */
  gint dialog_width;
  gboolean auto_resize;

  struct fontsel_info *fontsel_info;
};

struct _KryFontSelectionDialogClass
{
  GtkDialogClass parent_class;

  /* Padding for future expansion */
  void (*_gtk_reserved1) (void);
  void (*_gtk_reserved2) (void);
  void (*_gtk_reserved3) (void);
  void (*_gtk_reserved4) (void);
};



/*****************************************************************************
 * KryFontSelection functions.
 *   see the comments in the KryFontSelectionDialog functions.
 *****************************************************************************/

GType	   kry_font_selection_get_type		(void) G_GNUC_CONST;
GtkWidget *kry_font_selection_new (struct fontsel_info *fontsel_info);
gchar*	   kry_font_selection_get_font_name	(KryFontSelection *fontsel);

#ifndef GTK_DISABLE_DEPRECATED
GdkFont*   kry_font_selection_get_font		(KryFontSelection *fontsel);
#endif /* GTK_DISABLE_DEPRECATED */

gboolean              kry_font_selection_set_font_name    (KryFontSelection *fontsel,
							   const gchar      *fontname);
G_CONST_RETURN gchar* kry_font_selection_get_preview_text (KryFontSelection *fontsel);
void                  kry_font_selection_set_preview_text (KryFontSelection *fontsel,
							   const gchar      *text);

/*****************************************************************************
 * KryFontSelectionDialog functions.
 *   most of these functions simply call the corresponding function in the
 *   KryFontSelection.
 *****************************************************************************/

GType	   kry_font_selection_dialog_get_type	(void) G_GNUC_CONST;
GtkWidget* kry_font_selection_dialog_new	(const gchar	  *title, struct fontsel_info * fontsel_info);

/* This returns the X Logical Font Description fontname, or NULL if no font
   is selected. Note that there is a slight possibility that the font might not
   have been loaded OK. You should call kry_font_selection_dialog_get_font()
   to see if it has been loaded OK.
   You should g_free() the returned font name after you're done with it. */
gchar*	 kry_font_selection_dialog_get_font_name    (KryFontSelectionDialog *fsd);

#ifndef GTK_DISABLE_DEPRECATED
/* This will return the current GdkFont, or NULL if none is selected or there
   was a problem loading it. Remember to use gdk_font_ref/unref() if you want
   to use the font (in a style, for example). */
GdkFont* kry_font_selection_dialog_get_font	    (KryFontSelectionDialog *fsd);
#endif /* GTK_DISABLE_DEPRECATED */

/* This sets the currently displayed font. It should be a valid X Logical
   Font Description font name (anything else will be ignored), e.g.
   "-adobe-courier-bold-o-normal--25-*-*-*-*-*-*-*" 
   It returns TRUE on success. */
gboolean kry_font_selection_dialog_set_font_name    (KryFontSelectionDialog *fsd,
						     const gchar	*fontname);

/* This returns the text in the preview entry. You should copy the returned
   text if you need it. */
G_CONST_RETURN gchar* kry_font_selection_dialog_get_preview_text (KryFontSelectionDialog *fsd);

/* This sets the text in the preview entry. It will be copied by the entry,
   so there's no need to g_strdup() it first. */
void	 kry_font_selection_dialog_set_preview_text (KryFontSelectionDialog *fsd,
						     const gchar	    *text);



struct fontsel_info
{
  char *font_name;
  int font_size;
  gboolean bold;
  gboolean italic;
  gboolean strikethough;
  gboolean underline;
};


#endif /* __GTK_FONTSEL_H__ */
